/*****************************************************************************
*
* File              : TWI_vnc_slave.c
*							 01.05.09, TAB = 3
* AppNote           : AVR311 / 315
*
*
* nderungen        : Enthlt die Fuktionalit eines Slaves.
*							 Luft als (vereinfachter) Slave unabhngig von main():
*							 Es sind nur die Empfangsroutinen aktiv.
*							 Daten, die empfangen werden, werden direkt in einen Ringpuffer beschrieben.
*
****************************************************************************/

#include <avr/interrupt.h>              
#include <avr/io.h>
#include <avr/eeprom.h>
#include <inttypes.h>
#include "TWI_vnc_slave.h"
#include "global.h"

// Verwendet das globale Array vnc_buffer[] mit den Index vnc_head
// und einem Counter fr eingegangene Bytes vnc_bytes_in_buffer

// ----------------------------------------------------------------------------
// Initailisierung fr den Slave Modus
void TWI_init(void)
{
	TWAR = TWI_SLA_ADR;
   TWBR = TWI_TWBR;                             // Set bit rate register (Baudrate). Defined in header file.
// TWSR = TWI_TWPS;                             // Not used. Driver presumes prescaler to be 00.
//	TWDR = 0xFF;                                 // Default content = SDA released.
	TWCR = (1<<TWEN)|                            // TWI Interface enabled.
          (1<<TWIE)|(1<<TWINT)|                 // Enable TWI Interupt and clear the flag.
          (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|      // Prepare to ACK next time the Slave is addressed.
          (0<<TWWC); 
}

/****************************************************************************
This function is the Interrupt Service Routine (ISR), and called when the TWI interrupt is triggered;
that is whenever a TWI event has occurred. This function should not be called directly from the main
application.
****************************************************************************/

ISR(TWI_vect)
{
	static uint8_t idx;
	switch (TWSR)
	{
   case TWI_STX_ADR_ACK:            				// Own SLA+R has been received; ACK has been returned
		idx = 0;                       				// buffer pointer immer auf das 1. Element

// der Slave hat ein Byte gesendet, der Master mit ACK quittiert
   case TWI_STX_DATA_ACK:           				// Data byte in TWDR has been transmitted; ACK has been received
																// Es werden solange Daten gesendet, bis der Master mit NACK beendet
																// damit kann der Pointer aus dem Array hinauslaufen. 
		if (idx < 4) 
			{
			TWDR = VNC_tx.byte[idx % 0x04];			// gibt binr die ANzahl der bertragenen Datenblcke zurck
			idx++;
			}

		else if (idx > 3 && idx < 10)					// gibt die Zeit zurck
			{
			TWDR = time[idx - 4];						// bei der Jahresangabe muss 20 subtrahiert werden !!
			idx++;
			}

		else TWDR = 255;									// alle weiteren Anfragen werden mit 0xFF beantwortet !

      TWCR =(1<<TWEN)|                         	// TWI Interface enabled
            (1<<TWIE)|(1<<TWINT)|              	// Enable TWI Interupt and clear the flag to send byte
            (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   	// Send ACK
            (0<<TWWC);                         	//  
      break;


// TWI Slave Receiver status codes					// ###############################################

// der Slave ist im read-Modus und hat gerade seine eigene Adresse empfangen
// eigene Slave-Adresse + write empfangen und mit ACK quittiert
   case TWI_SRX_ADR_ACK: 								// Own SLA+W has been received ACK has been returned
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                         // 
		
		break;
			
// Slave im read Modus, Byte empfangen und mit ACK quittiert, nun das Byte ablegen
   case TWI_SRX_ADR_DATA_ACK:       				// Previously addressed with own SLA+W; data has been received; 
																// ACK has been returned
		vnc_buffer[vnc_head++] = TWDR;				// ein Byte wurde empfangen und in den Puffer eingereiht.
		if (vnc_head > (VNC_BUFFER_SIZE -1)) vnc_head = 0; // auf berlauf testen
		vnc_bytes_in_buffer++;							// Counter incrementieren
		if (vnc_head == vnc_tail) close_file(VNC_CLOSE);	// Notbremse, wenn der Puffer von hinten her berrollt wird
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                         //  
		break;

// im Slave-Modus laufen alle anderen Flle ber default:
   default:     
     
		TWCR = (1<<TWEN)|                         // TWI Interface enabled
             (1<<TWIE)|(1<<TWINT)|              // Enable TWI Interupt and clear the flag to send byte
             (1<<TWEA)|(0<<TWSTA)|(0<<TWSTO)|   // Send ACK after next reception
             (0<<TWWC);                      
		break; 
	}
}

// -- [ eof ] -----------------------------------------------------------------