// --- [ logger.c ] -----------------------------------------------------------
//

#define EXTERN

#include <avr/io.h>
#include <string.h>	
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "vnc_spi.h"
#include "rs232.h"
#include "spi_13bit.h"
#include "spi_hard.h"
#include "global.h"
#include "TWI_vnc_slave.h"

// --- public - Variablen -----------------------------------------------------

#define LOG_TWI									// TWI   loggen
#define LOG_RS232									// RS232 loggen
//#define LOG_SPI									// SPI   loggen

void check_time(void);							// beim Programmstart kann die Systemzeit gendert werden
void led_blink(uint8_t dauer, uint8_t cnt);// Routine zum mehrmaligen Blinkenlassen der Kontroll-LED
extern char VNC_filename[];

// ----------------------------------------------------------------------------
int main(void)
{
uint8_t x, cnt = 0, j = 0;

// Der Timer0 wird in VNC_spi_init() initialisiert
// Die Interuptroutine steht in der Datei 'vnc_spi.c'

// Eine Vorgabe fr die Systemzeit (wird verwendet fr die Dateizeit)
time[0] = 00;										// 00s
time[1] = 00;										// 00m
time[2] = 20;										// 20h
time[3] = 2;										// 02.
time[4] = 11;										// November
time[5] = 29;										// (2009 - 1980) = 29

LED_DDR  = 1<<LED;								// Port fr LED als Ausgang
LED_OFF;												// LED abschalten

// fr einen Taster zum Starten/Stoppen des Loggens
KEY_PORT |= ( 1<<KEY_START | 1<< KEY_CHANGE_ADR);// Pullups setzen

RS232_init();										// RS232 initialisieren
SPI_init();
TWI_init();

asm volatile ("sei");							// fr Timer und auch RX/TWI/SPI erforderlich

VNC_spi_init();									// Initialisierung und einiges mehr
LED_OFF;

// Wenn der Logger weiter unten zum 1. Mal bereit ist und auf das Startzeichen
// wartet, kann durch das Senden von 6 Byte die Uhrzeit verstellt werden.
// HIer wird festgelegt, ber welche Kanle der Logger beim Warten auf
// eine Eingabe zur Zeitverstellung reagiert.
// Auf dem gewhlten Kanal drfen zu diesem Zeitpunkt aber keine Daten
// anliegen, sonst knnten diese als Zeiteintrag interpretiert werden.

// die nachfolgenden Eintrge legen fest, ber welche Kanle (falls sie aktiv sind)
// die Systemzeit gendert werden kann.
// Das betrifft nur den Zeitbereich bis zum 1. Starten des Loggers, von da an
// kann die Zeit nicht mehr verndert werden.
// Auf den freigegebenen Kanlen drfen bis zum 1. Starten des Loggers keine 
// anderen Daten eingehen (die wrden dann als Zeitangabe interpretiert) !

// Zur besseren Kontrolle wird ein Pin zur Abfrage genutzt, ob das Datum
// gendert werden darf.
// Nur wenn der Jumper/Schalter PD.3 auf Masse zieht, wird der Empfang von Daten
// in der Wartephase enabled

if (KEY_CHANGE_ADR_PRESSED)					// Empfang whrend des Wartens aktivieren, wenn Pin auf gnd gezogen
{
#ifdef LOG_RS232
	RX_ENABLE;										// Systemzeit via RX ndern 
#endif
#ifdef LOG_TWI 
	TWI_ENABLE; 									// Systemzeit via TWI ndern
#endif
#ifdef LOG_SPI
//	SPI_ENABLE; 									// Systemzeit via SPI ndern
#endif
//led_blink(7,12);

put_s("time change enable");
put_c(13);
}

// --- falls beim Start kein Stick gefunden wird -------------------------------

if (VNC_check_stick_removed())				// wenn der Stick nicht gefunden wird
	{
	while(1)
		{
		led_blink(7,2);							// langsam 2x Blinken, dann den Stick testen
		if (VNC_check_stick_inserted())	break;// nachsehen, ob der Stick eingesteckt wurde
		}												// wenn ja, die Schleife verlassen
	}
														// hier angekommen ist ein Stick gefunden
														
while (VNC_spi_read()) _delay_ms(1); 		// alles auslesen, falls der VDIP noch auf Lager hat
														
														// Workaround:
VNC_file_access(FILE_WRITE);					// ohne liefert nach dem Start VNC_get_filesize() falsche Werte
VNC_file_access(FILE_CLOSE);					// und es wird trotz eines FILE_CLOSE
_delay_ms(10);										// beim nachfolgenden FILE_OPEN der Fehler
VNC_file_access(FILE_CLOSE);					// 'FO' gemeldet

/*
for (j = 0; j< 10; j++)							// zum Testen von VNC_get_filesize()
{
VNC_make_name(j, FILE);
VNC_get_filesize();
put_s(VNC_filename);
put_c(' ');
put_s(ultoa(VNC_fs.dword[0], msg,10));
put_c(13);
}
*/

// Zum Testen:
// Kopiert die Datei 'file_100.txt' (die muss vorhanden sein) auf die datei 'file_110.txt'
// Die Zieldatei ohne Rckfrage wird berschrieben !!
/*
 time[0] = 0;										// die Sekunden auf 0 stellen, damit die Dauer der Aktion
 VNC_copy_file(100,110);						// nachher in der File-time sichtbar wird
 put_s(utoa(VNC_get_error(), msg, 10));
 while(1);											// und beenden
*/
// --- ussere Hauptschleife ----------------------------------------------------------
//     hier ist der Stick prsent
//     Vorbereitung: den nchsten freien Dateinamen suchen,
//     dann Warten auf Startzeichen/Tastendruck - und das Wechseln des Sticks zulassen


while(1)
	{													// den nchsten unbenutzten Dateinamen suchen
	j = 0;											// wir beginnen bei 0 wie FILE_000.TXT
	VNC_make_name(j, FILE);						// erstellt nur den Dateinnamen
	while (VNC_file_exists())					// solange wie die Datei bereits besteht
		{												// weiter nach der nchsten Datei suchen, dazu
		j++;											// den Zhler incrementieren
		if (j == 255) error_msg(1);			// damit keine Endlosschleife entstehen kann
		VNC_make_name(j, FILE);					// und den nchsten Dateinamen generieren
		}	
														// jetzt ist VNC_error == 'CF' wegen fehlender Datei !!

	LED_ON;											// LED auf Dauerlicht

	vnc_head = 0;									// alle Pointer gehen in Ausgangsposition
	vnc_tail = 0;									// hier fr den Fall, dass die Systemzeit gendert werden soll
	vnc_bytes_in_buffer = 0;
														// Whrend der folgenden Schleife kann einmalig beim Programmstart
														// die Systemzeit verstellt werden !!
														// Die Zeit muss binr in 6 Byte definiert werden (-> check_time()) !
														
	x = 1;											// damit mehrere Schleifen gleichzeitig beendet werden knnen
														// hier Rcksprung zu while(1)
														// warten, solange wie PIND high (Taste nicht bettigt)
	while (KEY_START_RELEASED && x)			// whrend dieser Zeit besteht die Option, 
		{												// die Uhrzeit der Uhr zu verndern
														// Voraussetzung ist, dass der Empfang berhaupt enabled ist !
		if (vnc_bytes_in_buffer == 6)			// Dazu muss: Sekunde, Minute, Stunde, Tag, Monat, Jahr
			{											// als 6 Datenbyte an den Logger gesendet werden.
			check_time();							// hier wird die Eingabe auf Plausibilitt geprft
			vnc_head = 0;							// alles geht wieder in Grundposition
			vnc_bytes_in_buffer = 0;			// fr den Fall, dass noch ein weiteres Mal korrigiert werden soll	
			}
		// es knnte ja jetzt auch der Stick noch entfernt werden - daher prfen
		// allerdings ist bei einem anderen Stick der gewhlte freie Dateiname nicht mehr korrekt
		if(VNC_check_stick_removed())			// testen, ob der Stick entfernt wurde
			{
			while(x)									// Schleife bis ein Stick wieder gefunden wird
				{
				led_blink(3,1);					// schnelles Blinken mit ca. 5Hz
				if (VNC_check_stick_inserted())	x = 0;// prfen, ob der Stick wieder gefunden wurde	
				}										// x = 0 damit die while() Schleifen verlassen werden
			}					
		}
	
	if (x == 0) continue;						// an den Beginn der ueren Hauptschleife springen
														// Die Schleife ist durch Drcken der Taste verlassen worden,
	VNC_timeout = 5;								// nun 1/6 Sekunden warten zum Entprellen					
	while (VNC_timeout);
	while KEY_START_PRESSED;					// warten, bis die Taste losgelassen wird
	
	vnc_head = 0;									// alle Pointer gehen in Ausgangsposition
	vnc_tail = 0;
	vnc_bytes_in_buffer = 0;
	
	VNC_tx.dword[0] = 0;							// Counter fr die Anzahl der gesendeten Bytes
	cnt = 0;											// Counter fr die Anzahl der gesendeten Blcke
	VNC_file_access(FILE_WRITE);				// Datei zum Schreiben ffnen						

#ifdef LOG_RS232
	RX_ENABLE;										// Datenannahme via RX  enablen 
#endif
#ifdef LOG_TWI 
	TWI_ENABLE; 									// Datenannahme via TWI enablen
#endif
#ifdef LOG_SPI
	SPI_ENABLE; 									// Datenannahme via SPI enablen
#endif


// --- innere Hauptschleife ----------------------------------------------------------

	while(1)											// Schleife fr den Empfang von Daten
		{
		if (!(LED_timeout)) 						// nach 1 Sekunde ist VNC_timeout vom Timer auf 0 decrementiert
			{
			LED_timeout = 30;						// LED blinkt mit 0.5 Hz whrend des Loggens
			LED_TOG;									
			}
			
		if (vnc_bytes_in_buffer > VNC_BLOCK_SIZE) // wenn genug Daten im Puffer
			{
			asm volatile ("cli");				// whrend der Subtraktion die Interrupts sperren
			vnc_bytes_in_buffer -= VNC_BLOCK_SIZE;
			asm volatile ("sei");
		
			if (!(VNC_write_data(VNC_BLOCK_SIZE)))  error_msg(3);// Wenn ein Fehler vorliegt beenden

			VNC_tx.dword[0]+= VNC_BLOCK_SIZE;// Counter fr Gesamtzahl der geschriebenen Bytes
			cnt++;									// Zhler fr geschriebene Blcke
			}
														// Bei langsamen Logvorgngen zwischendrin die Datei 
														// in sinnvollen Abstnden schlieen,
														// bei ffnen erhlt sie nebenbei den neuen Zeitstempel
/*														// nach 20 Blcken = 10Kb 
		if (cnt > 20)								// Datei schlieen und gleich wieder ffnen
			{											// der Empfang luft dabei weiter
			cnt = 0;
														// Datei schlieen, bei Fehler beenden
			if (!(VNC_file_access(FILE_CLOSE)) error_msg(4);
														// Datei zum Schreiben ffnen, bei Fehler beenden
			if (!(VNC_file_access(FILE_WRITE)) error_msg(4);
			}
*/														// ------------------------------------------
														// Wenn die Taste bettigt wird, dann wird die geffnet
		if (KEY_START_PRESSED)					// Datei geschlossen und auf die Entnahme des Stick gewartet
			{
			close_file(VNC_CONTINUE);			// die Datei schlieen und weitermachen

			x = 1;									// Variable x, um mehrere Schleifen gleichzeitig zu verlassen
			while(x)									// Schleife zum Abziehen des Sticks, Neuanstecken des Sticks
				{										// kann durch einen Tastendruck bergangen werden
				if (!(LED_timeout))				// Ist hier anders gelst, weil die Taste gepollt wird
					{									// whrend der Ausfhrung von led_blink hat die Taste keine Funktion !
					LED_timeout = 7;
					LED_TOG;
					}
		
				if (KEY_START_PRESSED)			// wenn Taste bettigt wird, dann nicht weiter auf 
					{									// die Entnahme des Sticks warten
					VNC_timeout = 5;				// 1/6 Sekunden warten						
					while (VNC_timeout);
					while KEY_START_PRESSED;	// warten bis die Taste losgelassen wird
					x = 0;
					}

				if(VNC_check_stick_removed())	// testen, ob der Stick entfernt wurde
					{
					while(x)							// Schleife bis ein Stick wieder gefunden wird
						{
						led_blink(3,1);			// schnelles Blinken mit ca. 5Hz
						if (VNC_check_stick_inserted())	x = 0;// prfen, ob der Stick wieder gefunden wurde	
						}								// damit die while() Schleife verlassen wird
					}
				}
			LED_OFF;
			break;									// die innere Schleife verlassen
			}
		if (VNC_get_error()) error_msg(5);	// Wenn ein Fehler vorliegt, dann beenden
		}
	}
}

// ----------------------------------------------------------------------------
// Schreibt ungesicherte Daten in die Datei und schliet sie
// wenn action == VNC_CONTINUE wird weitergemacht,
// wenn action == VNC_CLOSE    dann wird Fehler gemeldet (Pufferberlauf aus ISR)
void close_file(uint8_t action)
{
	#ifdef LOG_RS232 
		RX_DISABLE;  								// keine Daten mehr via RS232 annehmen 
	#endif
	#ifdef LOG_TWI
		TWI_DISABLE; 								// keine Daten mehr via TWI annehmen
	#endif
	#ifdef LOG_SPI
		SPI_DISABLE; 								// keine Daten mehr via SPI annehmen
	#endif	
							
	VNC_timeout = 5;								// kurze Pause zum Entprellen
	while(VNC_timeout);		
	while KEY_START_PRESSED;					// warten, bis der Taster losgelassen wird	
	LED_ON;

	if (vnc_bytes_in_buffer)					// wenn noch neue Daten vorhanden, dann den Puffer leeren
		{
		VNC_write_data(vnc_bytes_in_buffer);
		VNC_tx.dword[0]+= vnc_bytes_in_buffer;// Counter fr Gesamtzahl der geschriebenen Bytes
		}

	VNC_file_access(FILE_CLOSE);				// Datei schlieen
	if (VNC_get_error()) error_msg(6);		// Wenn ein Fehler vorliegt, dann beenden
	LED_OFF;											// LED aus
	
	if (action == VNC_CLOSE) error_msg(2);	// Fehlermeldung ausgeben, wenn wegen Pufferberlauf aus
}														// empfangender Routine aufgerufen


// ----------------------------------------------------------------------------
// hier zeigt durch mehrfaches Blinken der LED den Fehlercode an:
// ca. 1 Sekunde AUS, dann Blinken mit ca. 2Hz
// die Anzahl der 'Blinks' gibt die Fehlernummer und die meldende Stelle an
void error_msg(uint8_t cnt)
{
LED_OFF;
while(1)
	{
	VNC_timeout = 33;								// ca. 1 Sekunde Pause (aus)
	while (VNC_timeout);
	
	led_blink(7, cnt);							// cnt x mit 2Hz blinken
	}
}

// ----------------------------------------------------------------------------
// lsst die LED cnt-mal blinken,
// die Frequenz hngt von dauer ab: Freq = (1 / (dauer * 2) * 1/30)
// z.B. 15 = 1 Hz
//       7 = 2 Hz
//       3 = 5 Hz
void led_blink(uint8_t dauer, uint8_t cnt)
{
while (cnt--)
	{
	LED_TOG;											// LED umschalten
	LED_timeout = dauer;
	while(LED_timeout);
	LED_TOG;											// LED alten Zustand wieder herstellen
	LED_timeout = dauer;
	while(LED_timeout);
	}
}

// ----------------------------------------------------------------------------
// Whrend des Wartens auf das 1. Startzeichen (NUR DANN - denn spter ist der Empfang
// whrend des Wartes deaktiviert) kann das Datum der Uhr gendert werden.
// Hier wird grob gecheckt, ob der Datums-/Zeiteintrag korrekt sein kann.
void check_time(void)
{
uint8_t i;

led_blink(3, 1);									// lt als Rckmeldung die LED 1x schnell blinken

if (vnc_buffer[0] > 59) return;				// Sekunden
if (vnc_buffer[1] > 59) return;				// Minuten
if (vnc_buffer[2] > 23) return;				// Stunden
if ((!(vnc_buffer[3])) || vnc_buffer[3] > 31) return; // Tag
if ((!(vnc_buffer[4])) || vnc_buffer[4] > 12) return; // Monat
if (vnc_buffer[5] > 15) return; 				// Jahr bis 2015

for (i = 0; i < 6; i++) time[i] = vnc_buffer[i];
time[5] += 20;										// beim Jahr noch 20 addieren (2009 - 1980) = 29
														// das Bezugsjahr ist 1980  = 0

led_blink(3, 5);									// lt als Rckmeldung die LED 5x schnell blinken
}														// insgesamt also 6x


// --- [ eof ] ----------------------------------------------------------------

