// --- [ spi_13bit.c ] --------------------------------------------------------
//
// tab = 3 
// ----------------------------------------------------------------------------

#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "spi_13bit.h"

extern uint8_t spi_in;						// die globale Variable transportiert eingehende Daten

// ----------------------------------------------------------------------------
// Schreibt ein Byte via SPI an einen VDIP1
// werden die Daten vom VDIP nicht quittiert, dann wird die Sendung wiederholt
// Rckgabe: TRUE, wenn das Byte erfolgreich abgeliefert wurde
//           FaLSE,  wenn das Byte (trotz mehrfacher Versuche) nicht angenommen wurde
//
//	Die bertragung eines Bytes dauert ca. 30us, die H/L Flanke steht nur 1 Takt lang auf HIGH

uint8_t VNC_spi_write(uint8_t spi_out)
{

uint8_t bitpos, cnt = 255;					// Anzahl der Schreibversuche bis zu einem Abbruch

do
  	{
	S_CS_VNC;        							// SPI-Interface aktiv
	bitpos = 0x80;								// fr jeden Rundgang neu initialisieren
  	S_SDI;           							// Startbit (immer 1)

  	SPI_PORT |=  (1<<SPI_SCLK);  R_SCLK;// fr 1. Flanke auf high setzen, danach nur noch togglen
  	R_SDI;           							// RW-Bit (0=Write)
  	S_SCLK;  R_SCLK; 							// H/L Flanke
  	R_SDI;           							// ADDR (0=Daten)
  	S_SCLK;  R_SCLK; 							// H/L Flanke

  	while ( bitpos )							// 8 Datenbits senden
  		{
    	if (spi_out & bitpos) { S_SDI;} else {R_SDI;}
    	S_SCLK;  R_SCLK; 						// H/L Flanke
		bitpos /= 2;
		}
  													// Statusbit einlesen, dazu bitpos verwenden
  	bitpos = SDO_VNC; 						// Statusbit (0=Accept, 1=Reject)
  	S_SCLK;  R_SCLK; 							// H/L Flanke
  													// End Sequenz
  	R_CS_VNC;        							// SPI-Interface inaktiv
  	S_SCLK;  R_SCLK;  						// H/L Flanke

//sei();
	cnt--;
	} while (bitpos && cnt);				// solange wie Daten abgewiesen werden und cnt > 0

if (bitpos) return (0); else return(1);// liefert TRUE falls erfolgreich, vertauscht 0/1
}													

// ----------------------------------------------------------------------------
// liest ein Byte via SPI vom VDIP
// die Daten werden ber die globale Variable spi_in weitergereicht.
// Rckgabe: TRUE, wenn ein neues Datenbyte empfangen wurde
//           FALSE, wenn alte Daten gelesen wurden, dann muss ein neuer Leseversuch gestartet werden.
uint8_t VNC_spi_read(void)
{
	uint8_t bitpos = 0x80;

  	S_CS_VNC;       							// SPI-Interface aktiv
	spi_in = 0;	
	
  	S_SDI;           							// Startbit (immer 1)

  	SPI_PORT |= (1<<SPI_SCLK);  R_SCLK; // fr 1. Flanke auf high setzen, danach nur noch togglen
  	S_SDI;           							// RW-Bit (1=Read)
  	S_SCLK;  R_SCLK; 							// H/L Flanke
  	R_SDI;           							// ADDR (0=Daten, 1=SPI-Statusinformation)
  	S_SCLK;  R_SCLK; 							// H/L Flanke
 
  	while ( bitpos )							// 8 Datenbits empfangen
		{
		if (SDO_VNC) spi_in |= bitpos;
    	S_SCLK;  R_SCLK; 						// H/L Flanke
		bitpos /= 2;
		}
  													// Statusbit einlesen, dazu bitpos verwerden
  	bitpos = SDO_VNC; 						// Statusbit (0=New Data, 1=Old Data)
  	S_SCLK;  R_SCLK; 							// H/L Flanke
  													// End Sequenz
  	R_CS_VNC;        							// SPI-Interface inaktiv
  	S_SCLK;  R_SCLK; 							// H/L Flanke

if (bitpos) return (0); else return(1);// liefert TRUE falls erfolgreich (vertauscht 0/1)
}	

// --- [ eof ] ----------------------------------------------------------------