// --- [ vnc_spi.h ] ----------------------------------------------------------
//
// tab = 3
// ----------------------------------------------------------------------------

#ifndef EXTERN
#define EXTERN extern
#endif

// --- von extern zugreifbare Funktionen --------------------------------------

void VNC_spi_init(void);							// Initialisierung
void VNC_firmware_version(void);
void VNC_set_filename(char *file);				// kopiert einen Textsting auf den Dateinamen (VNC_filename)
void VNC_make_name(uint8_t, uint8_t);			// legt einen Dateiname fest durch Anfgen von Ziffernfolgen
void VNC_copy_file(uint8_t, uint8_t);			// Kopiert eine Datei blockweise auf einen neuen Namen

uint8_t VNC_md(void);								// legt ein Verzeichnis an
uint8_t VNC_rd(void);								// lscht ein Verzeichnis (sofern es leer ist)
uint8_t VNC_cd(uint8_t);							// Wechsel zwischen Verzeichnissen

uint8_t VNC_file_access(uint8_t modus);		// ffnet und Schliet Dateien
uint8_t VNC_write_data(uint16_t cnt);			// schreibt einen Datenblock (vnc_buffer[])
uint8_t VNC_read_data(uint16_t cnt); 			// liest einen Datenblock
uint8_t VNC_seek(uint32_t pos);					// Positioniert innerhalb einer Datei
uint8_t VNC_get_filesize(void);					// holt die Dateigre einer Datei
uint8_t VNC_suspend(void);
uint8_t VNC_wake_up(void);

uint8_t VNC_file_exists(void);					// Prft, ob die Datei 'VNC_filename' vorhanden ist
uint8_t VNC_check_stick_removed(void);			// prft, ob der Stick noch anwesend ist (oder abgezogen wurde)	
uint8_t VNC_check_stick_inserted(void);		// prft, ob nach dem Abziehen des Sticks ein neuer gefunden wird	

uint8_t VNC_get_error(void);						// exportiert den aktuellen Errorcode aus dem Modul 

// --- Konstanten -------------------------------------------------------------

#define  VNC_RESET_PORT				PORTD
#define  VNC_RESET_DDR				DDRD
#define  VNC_RESET_PIN				PD2

#define  VNC_SPI_PORT				PORTC			// Port fr SPI-Kommunikation
#define	VNC_SPI_PIN					PINC
#define  VNC_SPI_DDR					DDRC
#define	VNC_SPI_CLK					PC3			// Clock (output)
#define	VNC_SPI_DI					PC2			// Data in (output)
#define	VNC_SPI_DO					PC1			// Data Out (input)
#define 	VNC_SPI_CS					PC0			// Chip Select (output)
															// VNC  Instructions
#define	VNC_SCS						0x10			// Konfiguration Short Command Set
#define	VNC_ECS						0x11			// Extended Command Set (default)
#define	VNC_IPA						0x90			// Monitor Mode ASCI
#define	VNC_IPH						0x91			// Monitor Mode Hex (default)

#define	VNC_SBD						0x14			// Set_Baud_Rate

#define	VNC_SUD						0x15			// Suspend disk
#define	VNC_WKD						0x16			// wake disk

#define	VNC_DIR						0x01			// DIR
#define	VNC_DLD						0x05			// Delete Directory
#define	VNC_MKD						0x06			// Make Directory
#define	VNC_CD						0x02			// Change Directory

#define	VNC_RD						0x04			// Read file (hole file)
#define	VNC_RDF						0x0B			// Read file (dword Bytes)
#define	VNC_DLF						0x07			// Delete File
#define	VNC_WRF						0x08			// write file
#define	VNC_OPW						0x09			// Open for write
#define	VNC_OPR						0x0E			// Open for read
#define	VNC_CLF						0x0A			// Close file
#define	VNC_REN						0x0C			// Rename file
#define	VNC_FS						0x12			// Free space (4 Byte)
#define	VNC_FSE						0x93			// Free Space (6 Byte)
#define	VNC_SEK						0x28			// Seek offset position
#define  VNC_EMPTY_CMD				0x00

#define	VNC_NO_ERR					0				// Erfolg oder Fehlercodes ...
#define	VNC_ERR_FI					1				// File invalid
#define	VNC_ERR_F0					2				// File open
#define	VNC_ERR_RO					4				// Read only
#define	VNC_ERR_CF					8				// Command Failed
#define	VNC_ERR_ND					16				// No Disk

#define	VNC_ERR_DF					32				// Disk full
#define	VNC_ERR_BC					128 + 32		// Bad Command
#define	VNC_ERR_NE					128 + 64		// Dir not empty
#define	VNC_ERR_TO					128 + 64 + 32// Timeout
#define	VNC_ERR						64 + 32		// sollte nicht auftreten

#define	VNC_BAD_COMMAND			0x42 + 0x43 + 0x0D	// BC
#define	VNC_COMMAND_FAILED		0x43 + 0x46 + 0x0D	// CF
#define  VNC_DISK_FULL				0x44 + 0x46 + 0x0D	// DF
#define	VNC_INVALID					0x46 + 0x49 + 0x0D	// FI
#define  VNC_READ_ONLY				0x52 + 0x4F + 0x0D	// RO
#define	VNC_FILE_OPEN				0x46 + 0x4F + 0x0D	// FO
#define	VNC_DIR_NOT_EMPTY			0x4E + 0x45 + 0x0D	// NE
#define	VNC_FILENAME_INVALID		0x46 + 0x4E + 0x0D	// FN
#define	VNC_NO_UPGRADE				0x4E + 0x55 + 0x0D 	// NU

#define	VNC_PROMPT_WAIT			10				// Wartezeit auf ein Prompt, wird von TIM0_ISR runtergezhlt
															// 
#define	FILE_READ					VNC_OPR		// Parameter der Funktion VNC_file_access(modus)
#define	FILE_WRITE					VNC_OPW
#define	FILE_CLOSE					VNC_CLF
#define	FILE_DELETE					VNC_DLF

#define	CHR_SYNCH					'E'			// Synch
#define	CHR_SPACE					0x20			// Leerzeichen
#define	CHR_RETURN					0x0D			// CR
#define	CHR_PROMPT					'>'			// Prompt

#define	TRUE							1
#define	FALSE							0
#define  FILE							0				// mit VNC_make_name() einen Filenamen generieren
#define  DIR							1				// mit VNC_make_name() einen Verzeichnisnamen generieren
#define  UP								1				// mit cd in bergeordnetes Verzeichnis wechseln
#define  DOWN							0				// mit cd in untergeordnetets Verzeichnis 'VNC_dirname' wechseln
#define  LOW_WORD						1				// 32 Bit - Filesize, Index
#define  HIGH_WORD					0

#define  DATA							0				// Parameter fr VNC_spi_read/write()
#define	STATUS						1				// Parameter fr VNC_spi_read/write()

// --- [ eof ] ----------------------------------------------------------------
